@@ -4,6 +4,7 @@
 static volatile im_slot_t slot_count = 1;
 static im_slot_destroy_t *volatile slot_destructors;
 static volatile i_mutex_t slot_mutex;
+static void fill_curves(im_context_t ctx);
 
 /*
 =item im_context_new()
@@ -45,13 +46,25 @@ im_context_new(void) {
     return NULL;
   }
 
+  ctx->file_magic = NULL;
+
+#if IMAGER_PTR_SIZE > 4
+  ctx->max_mmap_size = 0x80000000UL; /* 2GB */
+#else
+  ctx->max_mmap_size = 0x01000000UL; /* 16MB */
+#endif
+
   ctx->refcount = 1;
 
+  imcms_initialize(ctx);
+  ctx->rgb_profile = imcms_srgb_profile(ctx);
+  ctx->gray_profile = imcms_sgray_profile(ctx);
+  fill_curves(ctx);
+  
 #ifdef IMAGER_TRACE_CONTEXT
   fprintf(stderr, "im_context: created %p\n", ctx);
 #endif
 
-
   return ctx;
 }
 
@@ -105,6 +118,10 @@ im_context_refdec(im_context_t ctx, const char *where) {
   if (ctx->refcount != 0)
     return;
 
+  imcms_free_profile(ctx->rgb_profile);
+  imcms_free_profile(ctx->gray_profile);
+  imcms_finalize(ctx);
+
   /* lock here to avoid slot_destructors from being moved under us */
   i_mutex_lock(slot_mutex);
   for (slot = 0; slot < ctx->slot_alloc; ++slot) {
@@ -119,6 +136,18 @@ im_context_refdec(im_context_t ctx, const char *where) {
     if (ctx->error_stack[i].msg)
       myfree(ctx->error_stack[i].msg);
   }
+
+  {
+    im_file_magic *p = ctx->file_magic;
+    while (p != NULL) {
+      im_file_magic *n = p->next;
+      free(p->m.name);
+      free(p->m.magic);
+      free(p->m.mask);
+      free(p);
+      p = n;
+    }
+  }
 #ifdef IMAGER_LOG
   if (ctx->lg_file && ctx->own_log)
     fclose(ctx->lg_file);
@@ -191,8 +220,47 @@ im_context_clone(im_context_t ctx, const char *where) {
   nctx->max_height = ctx->max_height;
   nctx->max_bytes = ctx->max_bytes;
 
+  nctx->max_mmap_size = ctx->max_mmap_size;
+
   nctx->refcount = 1;
 
+  {
+    im_file_magic *inp = ctx->file_magic;
+    im_file_magic **outpp = &nctx->file_magic;
+    *outpp = NULL;
+    while (inp) {
+      im_file_magic *m = malloc(sizeof(im_file_magic));
+      if (!m) {
+	/* should free any parts of the list already allocated */
+	im_context_refdec(nctx, "failed cloning");
+	return NULL;
+      }
+      m->next = NULL;
+      m->m.name = strdup(inp->m.name);
+      m->m.magic_size = inp->m.magic_size;
+      m->m.magic = malloc(inp->m.magic_size);
+      m->m.mask = malloc(inp->m.magic_size);
+      if (m->m.name == NULL || m->m.magic == NULL || m->m.mask == NULL) {
+	free(m->m.name);
+	free(m->m.magic);
+	free(m->m.mask);
+	free(m);
+	im_context_refdec(nctx, "failed cloning");
+	return NULL;
+      }
+      memcpy(m->m.magic, inp->m.magic, m->m.magic_size);
+      memcpy(m->m.mask, inp->m.mask, m->m.magic_size);
+      *outpp = m;
+      outpp = &m->next;
+      inp = inp->next;
+    }
+  }
+
+  imcms_initialize(nctx);
+  nctx->rgb_profile = imcms_srgb_profile(nctx);
+  nctx->gray_profile = imcms_sgray_profile(nctx);
+  fill_curves(nctx);
+  
 #ifdef IMAGER_TRACE_CONTEXT
   fprintf(stderr, "im_context:%s: cloned %p to %p\n", where, ctx, nctx);
 #endif
@@ -297,3 +365,51 @@ im_context_slot_get(im_context_t ctx, im_slot_t slot) {
 
   return ctx->slots[slot];
 }
+
+/*
+=item im_add_file_magic(ctx, name, bits, mask, length)
+
+Add file type magic to the given context.
+
+=cut
+*/
+
+int
+im_add_file_magic(im_context_t ctx, const char *name,
+		  const unsigned char *bits, const unsigned char *mask,
+		  size_t length) {
+  im_file_magic *m = malloc(sizeof(im_file_magic));
+
+  if (m == NULL)
+    return 0;
+
+  if (length > 512)
+    length = 512;
+
+  m->m.name = strdup(name);
+  m->m.magic = malloc(length);
+  m->m.mask = malloc(length);
+  m->m.magic_size = length;
+
+  if (name == NULL || bits == NULL || mask == NULL) {
+    free(m->m.name);
+    free(m->m.magic);
+    free(m->m.mask);
+    free(m);
+    return 0;
+  }
+  memcpy(m->m.magic, bits, length);
+  memcpy(m->m.mask, mask, length);
+  m->next = ctx->file_magic;
+  ctx->file_magic = m;
+
+  return 1;
+}
+
+static void
+fill_curves(im_context_t ctx) {
+  if (!imcms_profile_curves(ctx->rgb_profile, ctx->rgb_curves, 3)
+      || !imcms_profile_curves(ctx->gray_profile, &ctx->gray_curve, 1)) {
+    fprintf(stderr, "Could not fetch profile curves for context\n");
+  }
+}
