@@ -1,6 +1,7 @@
 #define IMAGER_NO_CONTEXT
 #include "imager.h"
 #include "imageri.h"
+#include <assert.h>
 
 /*
 =head1 NAME
@@ -436,6 +437,8 @@ with the side of a filled area.
 i_fill_t *
 i_new_fill_hatch(const i_color *fg, const i_color *bg, int combine, int hatch, 
             const unsigned char *cust_hatch, i_img_dim dx, i_img_dim dy) {
+  assert(fg);
+  assert(bg);
   return i_new_hatch_low(fg, bg, NULL, NULL, combine, hatch, cust_hatch, 
                          dx, dy);
 }
@@ -464,6 +467,8 @@ with the side of a filled area.
 i_fill_t *
 i_new_fill_hatchf(const i_fcolor *fg, const i_fcolor *bg, int combine, int hatch, 
 		  const unsigned char *cust_hatch, i_img_dim dx, i_img_dim dy) {
+  assert(fg);
+  assert(bg);
   return i_new_hatch_low(NULL, NULL, fg, bg, combine, hatch, cust_hatch, 
                          dx, dy);
 }
@@ -472,6 +477,7 @@ static void fill_image(i_fill_t *fill, i_img_dim x, i_img_dim y,
 		       i_img_dim width, int channels, i_color *data);
 static void fill_imagef(i_fill_t *fill, i_img_dim x, i_img_dim y,
 			i_img_dim width, int channels, i_fcolor *data);
+static void destroy_image(i_fill_t *fill);
 struct i_fill_image_t {
   i_fill_t base;
   i_img *src;
@@ -486,7 +492,7 @@ image_fill_proto =
     {
       fill_image,
       fill_imagef,
-      NULL
+      destroy_image
     }
   };
 
@@ -530,6 +536,7 @@ i_new_fill_image(i_img *im, const double *matrix, i_img_dim xoff, i_img_dim yoff
   }
   else
     fill->has_matrix = 0;
+  i_img_refcnt_inc(im);
 
   return &fill->base;
 }
@@ -642,25 +649,23 @@ i_new_hatch_low(const i_color *fg, const i_color *bg,
   i_fill_hatch_t *fill = mymalloc(sizeof(i_fill_hatch_t)); /* checked 14jul05 tonyc */
 
   *fill = hatch_fill_proto;
-  /* Some Sun C didn't like the condition expressions that were here.
-     See https://rt.cpan.org/Ticket/Display.html?id=21944
-   */
-  if (fg)
+  if (fg && bg) {
     fill->fg = *fg;
-  else
-    fill->fg = fcolor_to_color(ffg);
-  if (bg)
     fill->bg = *bg;
-  else
-    fill->bg = fcolor_to_color(fbg);
-  if (ffg) 
-    fill->ffg = *ffg;
-  else
     fill->ffg = color_to_fcolor(fg);
-  if (fbg)
-    fill->fbg = *fbg;
-  else
     fill->fbg = color_to_fcolor(bg);
+  }
+  else if (ffg && fbg) {
+    fill->fg = fcolor_to_color(ffg);
+    fill->bg = fcolor_to_color(fbg);
+    fill->ffg = *ffg;
+    fill->fbg = *fbg;
+  }
+  else {
+    assert(0);
+    /* NOTREACHED */
+  }
+  
   if (combine) {
     i_get_combine(combine, &fill->base.combine, &fill->base.combinef);
   }
@@ -955,6 +960,12 @@ fill_imagef(i_fill_t *fill, i_img_dim x, i_img_dim y, i_img_dim width,
     i_adapt_fcolors(want_channels, f->src->channels, data, width);
 }
 
+static void
+destroy_image(i_fill_t *fill) {
+  struct i_fill_image_t *f = (struct i_fill_image_t *)fill;
+  i_img_destroy(f->src);
+}
+
 static void 
 fill_opacity(i_fill_t *fill, i_img_dim x, i_img_dim y, i_img_dim width,
 	     int channels, i_color *data) {
