@@ -1,11 +1,10 @@
 #include "imexttypes.h"
 #include "imager.h"
 #include "imio.h"
+#include "imexif.h"
 
 static im_context_t get_context(void);
 static i_img *mathom_i_img_8_new(i_img_dim, i_img_dim, int);
-static i_img *mathom_i_img_16_new(i_img_dim, i_img_dim, int);
-static i_img *mathom_i_img_double_new(i_img_dim, i_img_dim, int);
 static i_img *mathom_i_img_pal_new(i_img_dim, i_img_dim, int, int);
 static void mathom_i_clear_error(void);
 static void mathom_i_push_error(int, const char *);
@@ -40,19 +39,12 @@ im_ext_funcs imager_function_table =
     myfree_file_line,
     myrealloc_file_line,
 
-    mathom_i_img_8_new,
-    mathom_i_img_16_new,
-    mathom_i_img_double_new,
     mathom_i_img_pal_new,
     i_img_destroy,
     i_sametype,
-    i_sametype_chans,
+    i_sametype_chans_extra,
     i_img_info,
 
-    i_ppix,
-    i_gpix,
-    i_ppixf,
-    i_gpixf,
     i_plin,
     i_glin,
     i_plinf,
@@ -131,9 +123,6 @@ im_ext_funcs imager_function_table =
     /* IMAGER_API_LEVEL 3 functions */
     i_img_setmask,
     i_img_getmask,
-    i_img_getchannels,
-    i_img_get_width,
-    i_img_get_height,
     i_lhead,
     i_loog,
 
@@ -175,9 +164,6 @@ im_ext_funcs imager_function_table =
     io_glue_destroy,
 
     /* level 8 */
-    im_img_8_new,
-    im_img_16_new,
-    im_img_double_new,
     im_img_pal_new,
     im_clear_error,
     im_push_error,
@@ -213,9 +199,40 @@ im_ext_funcs imager_function_table =
     i_poly_aa_cfill_m,
 
     /* level 10 */
-    i_img_alpha_channel,
-    i_img_color_model,
-    i_img_color_channels
+    im_decode_exif,
+
+    /* level 11 */
+    im_io_set_max_mmap_size,
+    im_io_get_max_mmap_size,
+    i_img_refcnt_inc,
+    i_img_check_entries,
+    im_img_new,
+    im_img_8_new_extra,
+    im_img_16_new_extra,
+    im_img_double_new_extra,
+    i_new_image_data_alloc_def,
+    i_new_image_data_alloc_free,
+    myzmalloc,
+    myzmalloc_file_line,
+    i_gsampf_fp,
+    i_psampf_fp,
+    i_gsamp_bits_fb,
+    i_psamp_bits_fb,
+    i_img_data_fallback,
+    i_addcolors_forward,
+    i_getcolors_forward,
+    i_setcolors_forward,
+    i_colorcount_forward,
+    i_maxcolors_forward,
+    i_findcolor_forward,
+    i_gslin_fallback,
+    i_gslinf_fallback,
+    i_pslin_fallback,
+    i_pslinf_fallback,
+    im_img_float_new_extra,
+    im_lin_img_16_new_extra,
+    im_lin_img_double_new_extra,
+    im_fatal
   };
 
 /* in general these functions aren't called by Imager internally, but
@@ -226,151 +243,7 @@ im_ext_funcs imager_function_table =
    on the function names.
 */
 
-/*
-=item i_ppix(im, x, y, color)
-
-=category Drawing
-
-Sets the pixel at (x,y) to I<color>.
-
-Returns 0 if the pixel was drawn, or -1 if not.
-
-Does no alpha blending, just copies the channels from the supplied
-color to the image.
-
-=cut
-*/
-
-int 
-(i_ppix)(i_img *im, i_img_dim x, i_img_dim y, const i_color *val) {
-  return i_ppix(im, x, y, val);
-}
-
-/*
-=item i_gpix(im, C<x>, C<y>, C<color>)
-
-=category Drawing
-
-Retrieves the C<color> of the pixel (x,y).
-
-Returns 0 if the pixel was retrieved, or -1 if not.
-
-=cut
-*/
-
-int
-(i_gpix)(i_img *im,i_img_dim x,i_img_dim y,i_color *val) {
-  return i_gpix(im, x, y, val);
-}
-
-/*
-=item i_ppixf(im, C<x>, C<y>, C<fcolor>)
-
-=category Drawing
-
-Sets the pixel at (C<x>,C<y>) to the floating point color C<fcolor>.
-
-Returns 0 if the pixel was drawn, or -1 if not.
-
-Does no alpha blending, just copies the channels from the supplied
-color to the image.
-
-=cut
-*/
-int
-(i_ppixf)(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *val) {
-  return i_ppixf(im, x, y, val);
-}
-
-/*
-=item i_gpixf(im, C<x>, C<y>, C<fcolor>)
-
-=category Drawing
-
-Retrieves the color of the pixel (x,y) as a floating point color into
-C<fcolor>.
-
-Returns 0 if the pixel was retrieved, or -1 if not.
-
-=cut
-*/
-
-int
-(i_gpixf)(i_img *im,i_img_dim x,i_img_dim y,i_fcolor *val) {
-  return i_gpixf(im, x, y, val);
-}
-
-/*
-=item i_plin(im, l, r, y, colors)
-
-=category Drawing
-
-Sets (r-l) pixels starting from (l,y) using (r-l) values from
-I<colors>.
-
-Returns the number of pixels set.
-
-=cut
-*/
-
-i_img_dim
-(i_plin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals) {
-  return i_plin(im, l, r, y, vals);
-}
-
-/*
-=item i_glin(im, l, r, y, colors)
-
-=category Drawing
-
-Retrieves (r-l) pixels starting from (l,y) into I<colors>.
-
-Returns the number of pixels retrieved.
-
-=cut
-*/
-
-i_img_dim
-(i_glin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals) {
-  return i_glin(im, l, r, y, vals);
-}
-
-/*
-=item i_plinf(im, C<left>, C<right>, C<fcolors>)
-
-=category Drawing
-
-Sets (right-left) pixels starting from (left,y) using (right-left)
-floating point colors from C<fcolors>.
-
-Returns the number of pixels set.
-
-=cut
-*/
-
-i_img_dim
-(i_plinf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fcolor *vals) {
-  return i_plinf(im, l, r, y, vals);
-}
-
-/*
-=item i_glinf(im, l, r, y, colors)
-
-=category Drawing
-
-Retrieves (r-l) pixels starting from (l,y) into I<colors> as floating
-point colors.
-
-Returns the number of pixels retrieved.
-
-=cut
-*/
-
-i_img_dim
-(i_glinf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fcolor *vals) {
-  return i_glinf(im, l, r, y, vals);
-}
-
+#if 0
 /*
 =item i_gsamp(im, left, right, y, samples, channels, channel_count)
 
@@ -416,6 +289,7 @@ i_img_dim
            const int *chans, int chan_count) {
   return i_gsampf(im, l, r, y, samp, chans, chan_count);
 }
+#endif
 
 /*
 =item i_gsamp_bits(im, left, right, y, samples, channels, channel_count, bits)
@@ -638,11 +512,6 @@ mathom_i_img_8_new(i_img_dim xsize, i_img_dim ysize, int channels) {
   return i_img_8_new(xsize, ysize, channels);
 }
 
-static i_img *
-mathom_i_img_16_new(i_img_dim xsize, i_img_dim ysize, int channels) {
-  return i_img_16_new(xsize, ysize, channels);
-}
-
 static i_img *
 mathom_i_img_double_new(i_img_dim xsize, i_img_dim ysize, int channels) {
   return i_img_double_new(xsize, ysize, channels);
