@@ -15,6 +15,7 @@ maskimg.c - implements masked images/image subsets
 
 #include "imager.h"
 #include "imageri.h"
+#include "imapiver.h"
 
 #include <stdio.h>
 /*
@@ -36,14 +37,6 @@ typedef struct {
 #define MASKEXT(im) ((i_img_mask_ext *)((im)->ext_data))
 
 static void i_destroy_masked(i_img *im);
-static int i_ppix_masked(i_img *im, i_img_dim x, i_img_dim y, const i_color *pix);
-static int i_ppixf_masked(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *pix);
-static i_img_dim i_plin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals);
-static i_img_dim i_plinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fcolor *vals);
-static int i_gpix_masked(i_img *im, i_img_dim x, i_img_dim y, i_color *pix);
-static int i_gpixf_masked(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *pix);
-static i_img_dim i_glin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals);
-static i_img_dim i_glinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fcolor *vals);
 static i_img_dim i_gsamp_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample_t *samp, 
                           int const *chans, int chan_count);
 static i_img_dim i_gsampf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fsample_t *samp, 
@@ -57,36 +50,51 @@ static i_img_dim
 psampf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
 	       const i_fsample_t *samples, const int *chans, int chan_count);
 
+static i_img_dim i_gslin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample16_t *samp,
+                          int const *chans, int chan_count);
+static i_img_dim i_gslinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fsample_t *samp,
+                          int const *chans, int chan_count);
+static i_img_dim
+pslin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+             const i_sample16_t *samples, const int *chans, int chan_count);
+static i_img_dim
+pslinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+             const i_fsample_t *samples, const int *chans, int chan_count);
+static i_image_data_alloc_t *
+data_masked(i_img *im, i_data_layout_t layout, i_img_bits_t bits, unsigned flags,
+	    void **pdata, size_t *psize, int *pextra);
 /*
-=item IIM_base_masked
+=item vtable_masked
 
-The basic data we copy into a masked image.
+Virtual table for masked images.
 
 =cut
 */
-static i_img IIM_base_masked =
-{
-  0, /* channels set */
-  0, 0, 0, /* xsize, ysize, bytes */
-  ~0U, /* ch_mask */
-  i_8_bits, /* bits */
-  i_palette_type, /* type */
-  1, /* virtual */
-  NULL, /* idata */
-  { 0, 0, NULL }, /* tags */
-  NULL, /* ext_data */
 
-  i_ppix_masked, /* i_f_ppix */
-  i_ppixf_masked, /* i_f_ppixf */
-  i_plin_masked, /* i_f_plin */
-  i_plinf_masked, /* i_f_plinf */
-  i_gpix_masked, /* i_f_gpix */
-  i_gpixf_masked, /* i_f_gpixf */
-  i_glin_masked, /* i_f_glin */
-  i_glinf_masked, /* i_f_glinf */
+static const i_img_vtable
+vtable_mask = {
+  IMAGER_API_LEVEL,
+  
   i_gsamp_masked, /* i_f_gsamp */
   i_gsampf_masked, /* i_f_gsampf */
 
+  psamp_masked, /* i_f_psamp */
+  psampf_masked, /* i_f_psampf */
+
+  i_destroy_masked, /* i_f_destroy */
+
+  i_gsamp_bits_fb, /* i_f_gsamp_bits */
+  i_psamp_bits_fb, /* i_f_psamp_bits */
+
+  i_gslin_masked,
+  i_gslinf_masked,
+  pslin_masked,
+  pslinf_masked,
+
+  data_masked,
+
+  NULL, /* i_f_imageop */
+
   i_gpal_masked, /* i_f_gpal */
   i_ppal_masked, /* i_f_ppal */
   i_addcolors_forward, /* i_f_addcolors */
@@ -94,15 +102,7 @@ static i_img IIM_base_masked =
   i_colorcount_forward, /* i_f_colorcount */
   i_maxcolors_forward, /* i_f_maxcolors */
   i_findcolor_forward, /* i_f_findcolor */
-  i_setcolors_forward, /* i_f_setcolors */
-
-  i_destroy_masked, /* i_f_destroy */
-
-  NULL, /* i_f_gsamp_bits */
-  NULL, /* i_f_psamp_bits */
-
-  psamp_masked, /* i_f_psamp */
-  psampf_masked /* i_f_psampf */
+  i_setcolors_forward  /* i_f_setcolors */
 };
 
 /*
@@ -138,6 +138,10 @@ i_img_masked_new(i_img *targ, i_img *mask, i_img_dim x, i_img_dim y, i_img_dim w
     return NULL;
   }
   if (mask) {
+    if (mask->channels == 0) {
+      im_push_error(aIMCTX, 0, "mask has no color channels");
+      return NULL;
+    }
     if (w > mask->xsize)
       w = mask->xsize;
     if (h > mask->ysize)
@@ -153,15 +157,15 @@ i_img_masked_new(i_img *targ, i_img *mask, i_img_dim x, i_img_dim y, i_img_dim w
     return NULL;
   }
 
-  im = im_img_alloc(aIMCTX);
+  im = im_img_new(aIMCTX, &vtable_mask, w, h, targ->channels,
+                  targ->extrachannels, targ->bits);
+  if (im == NULL) {
+    return NULL;
+  }
 
-  memcpy(im, &IIM_base_masked, sizeof(i_img));
-  i_tags_new(&im->tags);
-  im->xsize = w;
-  im->ysize = h;
-  im->channels = targ->channels;
-  im->bits = targ->bits;
   im->type = targ->type;
+  im->isvirtual = 1;
+
   ext = mymalloc(sizeof(*ext));
   ext->targ = targ;
   ext->mask = mask;
@@ -172,6 +176,10 @@ i_img_masked_new(i_img *targ, i_img *mask, i_img_dim x, i_img_dim y, i_img_dim w
 
   im_img_init(aIMCTX, im);
 
+  i_img_refcnt_inc(targ);
+  if (mask)
+    i_img_refcnt_inc(mask);
+
   return im;
 }
 
@@ -188,279 +196,14 @@ Internal function.
 */
 
 static void i_destroy_masked(i_img *im) {
-  myfree(MASKEXT(im)->samps);
+  i_img_mask_ext *ext = MASKEXT(im);
+  i_img_destroy(ext->targ);
+  if (ext->mask)
+    i_img_destroy(ext->mask);
+  myfree(ext->samps);
   myfree(im->ext_data);
 }
 
-/*
-=item i_ppix_masked(i_img *im, i_img_dim x, i_img_dim y, const i_color *pix)
-
-Write a pixel to a masked image.
-
-Internal function.
-
-=cut
-*/
-static int i_ppix_masked(i_img *im, i_img_dim x, i_img_dim y, const i_color *pix) {
-  i_img_mask_ext *ext = MASKEXT(im);
-  int result;
-
-  if (x < 0 || x >= im->xsize || y < 0 || y >= im->ysize)
-    return -1;
-  if (ext->mask) {
-    i_sample_t samp;
-    
-    if (i_gsamp(ext->mask, x, x+1, y, &samp, NULL, 1) && !samp)
-      return 0; /* pretend it was good */
-  }
-  result = i_ppix(ext->targ, x + ext->xbase, y + ext->ybase, pix);
-  im->type = ext->targ->type;
-  return result;
-}
-
-/*
-=item i_ppixf_masked(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *pix)
-
-Write a pixel to a masked image.
-
-Internal function.
-
-=cut
-*/
-static int i_ppixf_masked(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *pix) {
-  i_img_mask_ext *ext = MASKEXT(im);
-  int result;
-
-  if (x < 0 || x >= im->xsize || y < 0 || y >= im->ysize)
-    return -1;
-  if (ext->mask) {
-    i_sample_t samp;
-    
-    if (i_gsamp(ext->mask, x, x+1, y, &samp, NULL, 1) && !samp)
-      return 0; /* pretend it was good */
-  }
-  result = i_ppixf(ext->targ, x + ext->xbase, y + ext->ybase, pix);
-  im->type = ext->targ->type;
-  return result;
-}
-
-/*
-=item i_plin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals)
-
-Write a row of data to a masked image.
-
-Internal function.
-
-=cut
-*/
-static i_img_dim i_plin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals) {
-  i_img_mask_ext *ext = MASKEXT(im);
-
-  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
-    if (r > im->xsize)
-      r = im->xsize;
-    if (ext->mask) {
-      i_img_dim i;
-      int simple = 0;
-      i_sample_t *samps = ext->samps;
-      i_img_dim w = r - l;
-
-      i_gsamp(ext->mask, l, r, y, samps, NULL, 1);
-      if (w < 10)
-        simple = 1;
-      else {
-        /* the idea is to make a fast scan to see how often the state
-           changes */
-        i_img_dim changes = 0;
-        for (i = 0; i < w-1; ++i)
-          if (!samps[i] != !samps[i+1])
-            ++changes;
-        if (changes > w/3) /* just rough */
-          simple = 1;
-      }
-      if (simple) {
-        /* we'd be calling a usually more complicated i_plin function
-           almost as often as the usually simple i_ppix(), so just
-           do a simple scan
-        */
-        for (i = 0; i < w; ++i) {
-          if (samps[i])
-            i_ppix(ext->targ, l + i + ext->xbase, y + ext->ybase, vals + i);
-        }
-        im->type = ext->targ->type;
-        return r-l;
-      }
-      else {
-        /* the scan above indicates there should be some contiguous 
-           regions, look for them and render
-        */
-        i_img_dim start;
-        i = 0;
-        while (i < w) {
-          while (i < w && !samps[i])
-            ++i;
-          start = i;
-          while (i < w && samps[i])
-            ++i;
-          if (i != start)
-            i_plin(ext->targ, l + start + ext->xbase, l + i + ext->xbase, 
-                   y + ext->ybase, vals + start);
-        }
-        im->type = ext->targ->type;
-        return w;
-      }
-    }
-    else {
-      i_img_dim result = i_plin(ext->targ, l + ext->xbase, r + ext->xbase, 
-                          y + ext->ybase, vals);
-      im->type = ext->targ->type;
-      return result;
-    }
-  }
-  else {
-    return 0;
-  }
-}
-
-/*
-=item i_plinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fcolor *vals)
-
-Write a row of data to a masked image.
-
-Internal function.
-
-=cut
-*/
-static i_img_dim i_plinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fcolor *vals) {
-  i_img_mask_ext *ext = MASKEXT(im);
-  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
-    if (r > im->xsize)
-      r = im->xsize;
-    if (ext->mask) {
-      i_img_dim i;
-      int simple = 0;
-      i_sample_t *samps = ext->samps;
-      i_img_dim w = r - l;
-
-      i_gsamp(ext->mask, l, r, y, samps, NULL, 1);
-      if (w < 10)
-        simple = 1;
-      else {
-        /* the idea is to make a fast scan to see how often the state
-           changes */
-        i_img_dim changes = 0;
-        for (i = 0; i < w-1; ++i)
-          if (!samps[i] != !samps[i+1])
-            ++changes;
-        if (changes > w/3) /* just rough */
-          simple = 1;
-      }
-      if (simple) {
-        /* we'd be calling a usually more complicated i_plin function
-           almost as often as the usually simple i_ppix(), so just
-           do a simple scan
-        */
-        for (i = 0; i < w; ++i) {
-          if (samps[i])
-            i_ppixf(ext->targ, l + i + ext->xbase, y + ext->ybase, vals+i);
-        }
-        im->type = ext->targ->type;
-        return r-l;
-      }
-      else {
-        /* the scan above indicates there should be some contiguous 
-           regions, look for them and render
-        */
-        i_img_dim start;
-        i = 0;
-        while (i < w) {
-          while (i < w && !samps[i])
-            ++i;
-          start = i;
-          while (i < w && samps[i])
-            ++i;
-          if (i != start)
-            i_plinf(ext->targ, l + start + ext->xbase, l + i + ext->xbase, 
-                    y + ext->ybase, vals + start);
-        }
-        im->type = ext->targ->type;
-        return w;
-      }
-    }
-    else {
-      i_img_dim result = i_plinf(ext->targ, l + ext->xbase, r + ext->xbase, 
-                           y + ext->ybase, vals);
-      im->type = ext->targ->type;
-      return result;
-    }
-  }
-  else {
-    return 0;
-  }
-}
-
-/*
-=item i_gpix_masked(i_img *im, i_img_dim x, i_img_dim y, i_color *pix)
-
-Read a pixel from a masked image.
-
-Internal.
-
-=cut
-*/
-static int i_gpix_masked(i_img *im, i_img_dim x, i_img_dim y, i_color *pix) {
-  i_img_mask_ext *ext = MASKEXT(im);
-
-  if (x < 0 || x >= im->xsize || y < 0 || y >= im->ysize)
-    return -1;
-
-  return i_gpix(ext->targ, x + ext->xbase, y + ext->ybase, pix);
-}
-
-/*
-=item i_gpixf_masked(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *pix)
-
-Read a pixel from a masked image.
-
-Internal.
-
-=cut
-*/
-static int i_gpixf_masked(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *pix) {
-  i_img_mask_ext *ext = MASKEXT(im);
-
-  if (x < 0 || x >= im->xsize || y < 0 || y >= im->ysize)
-    return -1;
-
-  return i_gpixf(ext->targ, x + ext->xbase, y + ext->ybase, pix);
-}
-
-static i_img_dim i_glin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals) {
-  i_img_mask_ext *ext = MASKEXT(im);
-  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
-    if (r > im->xsize)
-      r = im->xsize;
-    return i_glin(ext->targ, l + ext->xbase, r + ext->xbase, 
-                  y + ext->ybase, vals);
-  }
-  else {
-    return 0;
-  }
-}
-
-static i_img_dim i_glinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fcolor *vals) {
-  i_img_mask_ext *ext = MASKEXT(im);
-  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
-    if (r > im->xsize)
-      r = im->xsize;
-    return i_glinf(ext->targ, l + ext->xbase, r + ext->xbase, 
-                  y + ext->ybase, vals);
-  }
-  else {
-    return 0;
-  }
-}
-
 static i_img_dim i_gsamp_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample_t *samp, 
                           int const *chans, int chan_count) {
   i_img_mask_ext *ext = MASKEXT(im);
@@ -592,8 +335,8 @@ psamp_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
     else {
       result = i_psamp(ext->targ, l + ext->xbase, r + ext->xbase, 
 		       y + ext->ybase, samples, chans, chan_count);
-      im->type = ext->targ->type;
     }
+    im->type = ext->targ->type;
     ext->targ->ch_mask = old_ch_mask;
     return result;
   }
@@ -660,8 +403,8 @@ psampf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
       result = i_psampf(ext->targ, l + ext->xbase, r + ext->xbase, 
 			y + ext->ybase, samples,
 				 chans, chan_count);
-      im->type = ext->targ->type;
     }
+    im->type = ext->targ->type;
     ext->targ->ch_mask = old_ch_mask;
     return result;
   }
@@ -672,6 +415,179 @@ psampf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
   }
 }
 
+static i_img_dim
+i_gslin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample16_t *samp,
+               int const *chans, int chan_count) {
+  i_img_mask_ext *ext = MASKEXT(im);
+  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
+    if (r > im->xsize)
+      r = im->xsize;
+    return i_gslin(ext->targ, l + ext->xbase, r + ext->xbase,
+                   y + ext->ybase, samp, chans, chan_count);
+  }
+  else {
+    return 0;
+  }
+}
+
+static i_img_dim
+i_gslinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fsample_t *samp,
+               int const *chans, int chan_count) {
+  i_img_mask_ext *ext = MASKEXT(im);
+  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
+    if (r > im->xsize)
+      r = im->xsize;
+    return i_gslinf(ext->targ, l + ext->xbase, r + ext->xbase,
+                   y + ext->ybase, samp, chans, chan_count);
+  }
+  else {
+    return 0;
+  }
+}
+
+/*
+=item pslin_masked()
+
+i_pslin() implementation for masked images.
+
+=cut
+*/
+
+static i_img_dim
+pslin_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+             const i_sample16_t *samples, const int *chans, int chan_count) {
+  i_img_mask_ext *ext = MASKEXT(im);
+
+  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
+    unsigned old_ch_mask = ext->targ->ch_mask;
+    i_img_dim result = 0;
+    ext->targ->ch_mask = im->ch_mask;
+    if (r > im->xsize)
+      r = im->xsize;
+    if (ext->mask) {
+      i_img_dim w = r - l;
+      i_img_dim i = 0;
+      i_img_dim x = ext->xbase + l;
+      i_img_dim work_y = y + ext->ybase;
+      i_sample_t *mask_samps = ext->samps;
+
+      i_gsamp(ext->mask, l, r, y, mask_samps, NULL, 1);
+      /* not optimizing this yet */
+      while (i < w) {
+        if (mask_samps[i]) {
+          /* found a set mask value, try to do a run */
+          i_img_dim run_left = x;
+          const i_sample16_t *run_samps = samples;
+          ++i;
+          ++x;
+          samples += chan_count;
+
+          while (i < w && mask_samps[i]) {
+            ++i;
+            ++x;
+            samples += chan_count;
+          }
+          result += i_pslin(ext->targ, run_left, x, work_y, run_samps, chans, chan_count);
+        }
+        else {
+          ++i;
+          ++x;
+          samples += chan_count;
+          result += chan_count; /* pretend we wrote masked off pixels */
+        }
+      }
+    }
+    else {
+      result = i_pslin(ext->targ, l + ext->xbase, r + ext->xbase,
+                       y + ext->ybase, samples, chans, chan_count);
+    }
+    im->type = ext->targ->type;
+    ext->targ->ch_mask = old_ch_mask;
+    return result;
+  }
+  else {
+    dIMCTXim(im);
+    i_push_error(0, "Image position outside of image");
+    return -1;
+  }
+}
+
+/*
+=item pslinf_masked()
+
+i_pslinf() implementation for masked images.
+
+=cut
+*/
+
+static i_img_dim
+pslinf_masked(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y,
+             const i_fsample_t *samples, const int *chans, int chan_count) {
+  i_img_mask_ext *ext = MASKEXT(im);
+
+  if (y >= 0 && y < im->ysize && l < im->xsize && l >= 0) {
+    unsigned old_ch_mask = ext->targ->ch_mask;
+    i_img_dim result = 0;
+    ext->targ->ch_mask = im->ch_mask;
+    if (r > im->xsize)
+      r = im->xsize;
+    if (ext->mask) {
+      i_img_dim w = r - l;
+      i_img_dim i = 0;
+      i_img_dim x = ext->xbase + l;
+      i_img_dim work_y = y + ext->ybase;
+      i_sample_t *mask_samps = ext->samps;
+
+      i_gsamp(ext->mask, l, r, y, mask_samps, NULL, 1);
+      /* not optimizing this yet */
+      while (i < w) {
+        if (mask_samps[i]) {
+          /* found a set mask value, try to do a run */
+          i_img_dim run_left = x;
+          const i_fsample_t *run_samps = samples;
+          ++i;
+          ++x;
+          samples += chan_count;
+
+          while (i < w && mask_samps[i]) {
+            ++i;
+            ++x;
+            samples += chan_count;
+          }
+          result += i_pslinf(ext->targ, run_left, x, work_y, run_samps, chans, chan_count);
+        }
+        else {
+          ++i;
+          ++x;
+          samples += chan_count;
+          result += chan_count; /* pretend we wrote masked off pixels */
+        }
+      }
+    }
+    else {
+      result = i_pslinf(ext->targ, l + ext->xbase, r + ext->xbase,
+                       y + ext->ybase, samples, chans, chan_count);
+    }
+    im->type = ext->targ->type;
+    ext->targ->ch_mask = old_ch_mask;
+    return result;
+  }
+  else {
+    dIMCTXim(im);
+    i_push_error(0, "Image position outside of image");
+    return -1;
+  }
+}
+
+static i_image_data_alloc_t *
+data_masked(i_img *im, i_data_layout_t layout, i_img_bits_t bits, unsigned flags,
+	    void **pdata, size_t *psize, int *pextra) {
+  i_img_mask_ext *ext = MASKEXT(im);
+
+  im->type = ext->targ->type;
+
+  return i_img_data_fallback(im, layout, bits, flags, pdata, psize, pextra);
+}
 
 /*
 =back
