@@ -559,6 +559,7 @@ tga_palette_read(io_glue *ig, i_img *img, int bytepp, int colourmaplength) {
   palbuf   = mymalloc(palbsize);
   
   if (i_io_read(ig, palbuf, palbsize) != palbsize) {
+    myfree(palbuf);
     i_push_error(errno, "could not read targa colormap");
     return 0;
   }
@@ -754,7 +755,7 @@ i_readtga_wiol(io_glue *ig, int length) {
   
   img = mapped ? 
     i_img_pal_new(width, height, channels, 256) :
-    i_img_empty_ch(NULL, width, height, channels);
+    i_img_8_new(width, height, channels);
 
   if (!img) {
     if (idstring) 
